#include "stdafx.h"
#include "dialog.hpp"
#include "wll_spy.hpp"
#include "far.hpp"
#include "wndlist.hpp"
#include "common.hpp"
#include "options.hpp"
#include "msg.hpp"

struct DialogParams MainDlgParams = {
  70, // width
  21, // height

  70, // minwidth
  21, // minheight
  -1, // maxwidth
  21, // maxheight
};

const int WidthDependent[11] = {
  staticDialogBox, 
  cbWindowsList, edWindowsList, cbHistory,
  laSeparator01, laSeparator02, staticHelpBox,
  laHelp,
  edClassValue, edTextValue, cbRectValue
};

void SetWindowsListCaption(HANDLE hDlg, HWND hwnd)
{
  char buf[MAXSIZE];
  
  if (theSpyWindows->GetFormatString(hwnd, 
    theOptions->Value(opt_WLCaptionFormat)->StrValue, 
    &buf[0], sizeof(buf)))
  {
    SetDlgItemStr(hDlg, edWindowsList, buf);
  }
}

void FillDialog(HANDLE hDlg, struct SpyWindowInfo *windowInfo, BOOL bAddToHistory)
{
  static BOOL WorkAround_FirstFill = TRUE;

  char buf[MAXSIZE];

  Info.SendDlgMessage(hDlg, DM_ENABLEREDRAW, FALSE, 0);

  MkWindowDepends(hDlg, windowInfo->hwnd, cbDependsWindows);
  GetWindowInfoStr(windowInfo, SF_HWND, buf, sizeof(buf));
  SetDlgItemStr(hDlg, cbDependsWindows, buf);
  
  GetWindowInfoStr(windowInfo, SF_WINDOWTEXT, buf, sizeof(buf));
  SetDlgItemStr(hDlg, edTextValue, buf);

  GetWindowInfoStr(windowInfo, SF_CLASSNAME, buf, sizeof(buf));
  SetDlgItemStr(hDlg, edClassValue, buf);

  MkWindowStyle(hDlg, windowInfo->classStyle, STDClStyles, cbClassStyleValue);
  GetWindowInfoStr(windowInfo, SF_CLASSSTYE, buf, sizeof(buf));
  SetDlgItemStr(hDlg, cbClassStyleValue, buf);
  
  MkWindowStyle(hDlg, windowInfo->windowStyle, STDStyles, cbStyleValue);
  GetWindowInfoStr(windowInfo, SF_WINDOWSTYLE, buf, sizeof(buf));
  SetDlgItemStr(hDlg, cbStyleValue, buf);
  
  MkWindowStyle(hDlg, windowInfo->windowExStyle, STDExStyles, cbExStyleValue);
  GetWindowInfoStr(windowInfo, SF_WINDOWEXSTYLE, buf, sizeof(buf));
  SetDlgItemStr(hDlg, cbExStyleValue, buf);
  
  MkWindowRects(hDlg, windowInfo->hwnd, cbRectValue);
  GetWindowInfoStr(windowInfo, SF_RECTANGLE, buf, sizeof(buf));
  SetDlgItemStr(hDlg, cbRectValue, buf);
 
  GetWindowInfoStr(windowInfo, SF_THREADID, buf, sizeof(buf));
  SetDlgItemStr(hDlg, edThreadIDValue, buf);
  
  GetWindowInfoStr(windowInfo, SF_PROCESSID, buf, sizeof(buf));
  SetDlgItemStr(hDlg, edProcessIDValue, buf);

  static int ResetCursorPos[] = {
    cbDependsWindows, 
    edTextValue, edClassValue,
    cbClassStyleValue, cbStyleValue, cbExStyleValue,
    cbRectValue,
    edThreadIDValue, edProcessIDValue
  };

  for (int i = 0; i < sizeof(ResetCursorPos) / sizeof(ResetCursorPos[0]); i++)
  {
    COORD coord = {0, 0};
    if (!WorkAround_FirstFill)
      Info.SendDlgMessage(hDlg, DM_SETCURSORPOS, ResetCursorPos[i], (long)&coord);
    Info.SendDlgMessage(hDlg, DM_EDITUNCHANGEDFLAG, ResetCursorPos[i], 0);
   }
   WorkAround_FirstFill = FALSE;

  SetActions(hDlg, windowInfo->hwnd);
  
  struct FarListPos listPos;
  int CurPos = theSpyWindows->Find(wndInfo.hwnd);
  listPos.SelectPos = CurPos;
  listPos.TopPos = -1;
  Info.SendDlgMessage(hDlg, DM_LISTSETCURPOS, cbWindowsList, (long)&listPos);
  
  SetWindowsListCaption(hDlg, windowInfo->hwnd);

  if (bAddToHistory)
    AddToHistory(hDlg, windowInfo->hwnd);

  SetEnable(hDlg, windowInfo->hwnd);

  Info.SendDlgMessage(hDlg, DM_ENABLEREDRAW, TRUE, 0);
}

void AddToHistory(HANDLE hDlg, HWND hwnd)
{
  (void) hDlg;

  struct WindowHistoryItem historyItem;
  historyItem.hwnd = hwnd;
  theWindowsHistory->Add(&historyItem);
}

void SetEnable(HANDLE hDlg, HWND hwnd)
{

  DWORD res = theWindowsHistory->GetCurrentState();
  
  Info.SendDlgMessage(hDlg, DM_ENABLE, btnWindowPrev, !(res & WHR_ISFIRST));
  Info.SendDlgMessage(hDlg, DM_ENABLE, btnWindowSucc, !(res & WHR_ISLAST));

  Info.SendDlgMessage(hDlg, DM_ENABLE, btnBlinkWindow, IsWindowVisible(hwnd));
}

void SetActions(HANDLE hDlg, HWND hwnd)
{
  if (!theOptions->Value(opt_ALSmartSelect)->IntValue &&
      !theOptions->Value(opt_ALSmartEnable)->IntValue)
    return;
  
  BOOL isWindow = IsWindow(hwnd);

  if (theOptions->Value(opt_ALSmartEnable)->IntValue)
  {
    for(int i = lbActionsFirst; i <= lbActionsLast; i++)
      if (isWindow)
        ListItems[i - lbActionsFirst].Flags &= ~MIF_DISABLE;
      else
        ListItems[i - lbActionsFirst].Flags |= MIF_DISABLE;
    
    if (isWindow)
    {
      if (IsIconic(hwnd))
        ListItems[actionMinimize - lbActionsFirst].Flags = MIF_DISABLE;
      else
        if (IsZoomed(hwnd))
          ListItems[actionMaximize - lbActionsFirst].Flags = MIF_DISABLE;
        else
          ListItems[actionRestore - lbActionsFirst].Flags = MIF_DISABLE;
      
      if (IsWindowEnabled(hwnd))
        ListItems[actionEnable - lbActionsFirst].Flags = MIF_DISABLE;
      else
        ListItems[actionDisable - lbActionsFirst].Flags = MIF_DISABLE;
    }
  }
  
  if (theOptions->Value(opt_ALSmartSelect)->IntValue)
  {
    DialogControls action = ILast;
    
    if (isWindow)
    {
      if (IsWindowVisible(hwnd))
      {
        if (IsIconic(hwnd) || IsZoomed(hwnd))
          action = actionRestore;
        else
          if (ListItems[actionShow - lbActionsFirst].Flags == MIF_SELECTED)
            action = actionHide;
          else
            if (IsWindowEnabled(hwnd))
              action = actionDisable;
            else
              action = actionEnable;
      }
      else
        action = actionShow;
    }
    
    for (int i = lbActionsFirst; i <= lbActionsLast; i++)
      ListItems[i - lbActionsFirst].Flags &= ~MIF_SELECTED;

    if (isWindow)
      ListItems[action - lbActionsFirst].Flags = MIF_SELECTED;
  }
  
  Info.SendDlgMessage(hDlg, DM_LISTSET, lbActions, (long)&List);
}

void ResizeDialog(HANDLE hDlg, struct DialogParams *dlgParams, int newX, int newY, BOOL bCenterDialog)
{
  (void) newY;

  int i;
  FarDialogItem item;
  DialogParams params = *dlgParams;
  
  HANDLE hStdout; 
  CONSOLE_SCREEN_BUFFER_INFO csbiInfo; 

  hStdout = GetStdHandle(STD_OUTPUT_HANDLE);

  if (hStdout == INVALID_HANDLE_VALUE) 
    return; 

  if (!GetConsoleScreenBufferInfo(hStdout, &csbiInfo))
    return;

  if (params.minwidth < 0)
    params.minwidth = 0;
  if (params.minheight < 0)
    params.maxheight = 0;

  if (params.maxwidth == -1 || params.maxwidth > csbiInfo.dwSize.X)
    params.maxwidth = csbiInfo.dwSize.X;
  if (params.maxheight == -1 || params.maxheight > csbiInfo.dwSize.Y)
    params.maxheight = csbiInfo.dwSize.Y;

  COORD newCoord;

  newCoord.X = (SHORT)(newX);

  if (newCoord.X < params.minwidth) 
    newCoord.X = (SHORT)params.minwidth;
  if (newCoord.X > params.maxwidth) 
    newCoord.X = (SHORT)params.maxwidth;

  int dx = newCoord.X - params.width;

 Info.SendDlgMessage(hDlg, DM_ENABLEREDRAW, FALSE, 0);

 if (dx)
  {
    dlgParams->width = newCoord.X;
    newCoord.Y = (SHORT)params.height;

    if (dx > 0)
      Info.SendDlgMessage(hDlg, DM_RESIZEDIALOG, 0, (long)&newCoord);

    for (i = 0 ; i < sizeof(WidthDependent) / sizeof(WidthDependent[0]); i++)
    {
      Info.SendDlgMessage(hDlg, DM_GETDLGITEM, WidthDependent[i], (long)&item);
      item.X2 += dx;
      Info.SendDlgMessage(hDlg, DM_SETDLGITEM, WidthDependent[i], (long)&item);
    }

    if (dx < 0)
      Info.SendDlgMessage(hDlg, DM_RESIZEDIALOG, 0, (long)&newCoord);
  }

  if (bCenterDialog)
  {
    COORD coord = {-1, -1};
    Info.SendDlgMessage(hDlg, DM_MOVEDIALOG, TRUE, (long)&coord);
  }

 Info.SendDlgMessage(hDlg, DM_ENABLEREDRAW, TRUE, 0);
}