#include "stdafx.h"
#include "filter.hpp"
#include "far.hpp"
#include "msg.hpp"

const int FltW = 40;
const int FltH = 13;

void FltDlgSetEnable(HANDLE hDlg, int element)
{
  int comboList[] = 
  {
    FLTID(laFltHandle), 
    FLTID(laFltClass),
    FLTID(laFltText),
    FLTID(laFltThreadID),
    FLTID(laFltProcessID),
  };

  if (element == -1)
  {
    for (int i = 0; i < sizeof(comboList) / sizeof(comboList[0]); i++)
    {
      BOOL bCheked = Info.SendDlgMessage(hDlg, DM_GETCHECK, comboList[i], 0) == BSTATE_CHECKED;
      Info.SendDlgMessage(hDlg, DM_ENABLE, comboList[i] + 1, bCheked);
    }
  }
  else
  {
    BOOL bCheked = Info.SendDlgMessage(hDlg, DM_GETCHECK, element, 0) == BSTATE_CHECKED;
    Info.SendDlgMessage(hDlg, DM_ENABLE, element + 1, bCheked);
  }
}

void FltDlgReset(HANDLE hDlg, int element)
{
  int comboList[] = 
  {
    FLTID(laFltHandle), 
    FLTID(laFltClass),
    FLTID(laFltText),
    FLTID(laFltThreadID),
    FLTID(laFltProcessID),
  };

  FarDialogItemData itemData = {0, ""};


  if (element == -1)
  {
    for (int i = 0; i < sizeof(comboList) / sizeof(comboList[0]); i++)
    {
      Info.SendDlgMessage(hDlg, DM_SETCHECK, comboList[i], FALSE);
      Info.SendDlgMessage(hDlg, DM_SETTEXT, comboList[i] + 1, (long)&itemData);
    }
  }
  else
  {
    Info.SendDlgMessage(hDlg, DM_SETCHECK, element, FALSE);
    Info.SendDlgMessage(hDlg, DM_SETTEXT, element + 1, (long)&itemData);
  }

  FltDlgSetEnable(hDlg, element);
}

DWORD HexStrToDword(const char *str)
{
  int len = lstrlen(str);

  if (!str || len > 8)
    return 0;

  const DWORD radix = 16;

  DWORD result = 0;
  DWORD factor = 1;

  for (int i = len - 1; i >= 0; i--)
  {
    int digit;
    char ch = str[i];

    if (ch >= '0' && ch <= '9')
      digit = ch - '0';
    else
      if (ch >= 'a' && ch <= 'f')
        digit = ch - 'a' + 10;
      else
        if (ch >= 'A' && ch <= 'F')
          digit = ch - 'A' + 10;
        else
          return 0;

    result += digit * factor;
    
    factor *= radix;
  }
  
  return result;
}

long WINAPI FilterDlgProc(HANDLE hDlg, int Msg, int Param1, long Param2)
{
  switch (Msg) 
  {
  case DN_INITDIALOG:
    FltDlgSetEnable(hDlg, -1);
    return FALSE;

  case DN_BTNCLICK:
    switch (Param1)
    {
    case FLTID(laFltHandle):
    case FLTID(laFltClass):
    case FLTID(laFltText):
    case FLTID(laFltThreadID):
    case FLTID(laFltProcessID):
      FltDlgSetEnable(hDlg, Param1);
      return TRUE;

    case FLTID(btnFltReset):
      FltDlgReset(hDlg, -1);
      return FALSE;
    }
    break;

  case DN_CLOSE:
    return (Param1 == -1 || 
            Param1 == FLTID(btnFltOk) || 
            Param1 == FLTID(btnFltCancel));
    break;
  }
  
  return Info.DefDlgProc(hDlg, Msg, Param1, Param2);
}


BOOL SetFilter(struct SpyFilter *windowFilter)
{
  if (!windowFilter)
    return FALSE;

  struct FarDialogItem FilterDlgItems[fltDlgCount];

  const int cx = FltW / 2;
  const int lx = cx - 4;
  
  static const struct InitDialogItem FilterItems[] = {
    DI_DOUBLEBOX,   0,       0,     FltW-1, FltH-1,  0, staticFltDlgBox,
      
    DI_CHECKBOX,    2,       1,          0,      0,  0, laFltHandle,
    DI_FIXEDIT,  cx+1,       1,    cx+1+lx,      0,  DIF_MASKEDIT, -1,

    DI_SINGLEBOX,   1,       2,     FltW-2,      2,  0, -1,

    DI_CHECKBOX,    2,       3,          0,      0,  0, laFltClass,
    DI_EDIT,        2,       4,     FltW-3,      0,  0, -1,

    DI_CHECKBOX,    2,       5,          0,      0,  0, laFltText,
    DI_EDIT,        2,       6,     FltW-3,      0,  0, -1,

    DI_SINGLEBOX,   1,       7,     FltW-2,      7,  0, -1,

    DI_CHECKBOX,    2,       8,          0,      0,  0, laFltThreadID,
    DI_FIXEDIT,     2,       9,       2+lx,      0,  DIF_MASKEDIT, -1,
    
    DI_CHECKBOX, cx+1,       8,          0,      0,  0, laFltProcessID,
    DI_FIXEDIT,  cx+1,       9,    cx+1+lx,      0,  DIF_MASKEDIT, -1,
    
    DI_SINGLEBOX,   1,  FltH-3,     FltW-2, FltH-3,  0, -1,

    DI_BUTTON,      0,  FltH-2,          0,      0,  DIF_CENTERGROUP, btnFltOk,
    DI_BUTTON,      0,  FltH-2,          0,      0,  DIF_CENTERGROUP, btnFltReset,
    DI_BUTTON,      0,  FltH-2,          0,      0,  DIF_CENTERGROUP, btnFltCancel,
  };
  InitDialogItems(FilterItems, FilterDlgItems, sizeof(FilterItems) / sizeof(FilterItems[0]));
    
  static const char maskHexHandle[] = "HHHHHHHH";
  FilterDlgItems[FLTID(edFltHandle)].Mask = maskHexHandle;
  FilterDlgItems[FLTID(edFltThreadID)].Mask = maskHexHandle;
  FilterDlgItems[FLTID(edFltProcessID)].Mask = maskHexHandle;

  FilterDlgItems[FLTID(btnFltOk)].DefaultButton = TRUE;

  FilterDlgItems[FLTID(laFltHandle)].Selected = windowFilter->isHWND;
  wsprintf(FilterDlgItems[FLTID(edFltHandle)].Data, "%08X", windowFilter->hwnd);

  FilterDlgItems[FLTID(laFltClass)].Selected = windowFilter->isClassName;
  lstrcpyn(FilterDlgItems[FLTID(edFltClass)].Data, windowFilter->className, MAXSIZE);

  FilterDlgItems[FLTID(laFltText)].Selected = windowFilter->isWindowText;
  lstrcpyn(FilterDlgItems[FLTID(edFltText)].Data, windowFilter->windowText, MAXSIZE);

  FilterDlgItems[FLTID(laFltThreadID)].Selected = windowFilter->isThreadID;
  wsprintf(FilterDlgItems[FLTID(edFltThreadID)].Data, "%08X", windowFilter->ThreadID);
  
  FilterDlgItems[FLTID(laFltProcessID)].Selected = windowFilter->isProcessID;
  wsprintf(FilterDlgItems[FLTID(edFltProcessID)].Data, "%08X", windowFilter->ProcessID);

  int exitCode;
  exitCode = Info.DialogEx(Info.ModuleNumber, 
    -1, -1, 
    FltW, FltH,
    NULL, FilterDlgItems,
    sizeof(FilterDlgItems) / sizeof(FilterDlgItems[0]),
    0, 
    0, 
    FilterDlgProc, 0);

  if (exitCode == FLTID(btnFltOk))
  {
    ZeroMemory(windowFilter, sizeof(windowFilter));

    windowFilter->isHWND = FilterDlgItems[FLTID(laFltHandle)].Selected;
    windowFilter->hwnd = (HWND)HexStrToDword(FilterDlgItems[FLTID(edFltHandle)].Data);

    windowFilter->isClassName = FilterDlgItems[FLTID(laFltClass)].Selected;
    lstrcpyn(windowFilter->className, FilterDlgItems[FLTID(edFltClass)].Data, sizeof(windowFilter->className));

    windowFilter->isWindowText = FilterDlgItems[FLTID(laFltText)].Selected;
    lstrcpyn(windowFilter->windowText, FilterDlgItems[FLTID(edFltText)].Data, sizeof(windowFilter->windowText));

    windowFilter->isThreadID = FilterDlgItems[FLTID(laFltThreadID)].Selected;
    windowFilter->ThreadID = HexStrToDword(FilterDlgItems[FLTID(edFltThreadID)].Data);

    windowFilter->isProcessID = FilterDlgItems[FLTID(laFltProcessID)].Selected;
    windowFilter->ProcessID = HexStrToDword(FilterDlgItems[FLTID(edFltProcessID)].Data);

  }

  return (exitCode == FLTID(btnFltOk));
}

