#include "stdafx.h"
#include "windows.hpp"
#include "standart.hpp"

struct SpyFormat spyFormats[SF_LAST] =
{
"hwnd",           FT_INT, //  SF_HWND,
  
"parent",         FT_INT, //  SF_PARENT,
"owner",          FT_INT, //  SF_OWNER,
"hwndnext",       FT_INT, //  SF_HWNDNEXT,
"hwndfirst",      FT_INT, //  SF_HWNDFIRST,
"hwndlast",       FT_INT, //  SF_HWNDLAST,
"hwndprev",       FT_INT, //  SF_HWNDPREV,
"child",          FT_INT, //  SF_CHILD,

"windowtext",     FT_STR, //  SF_WINDOWTEXT,
"classname",      FT_STR, //  SF_CLASSNAME,

"classstyle",     FT_INT, //  SF_CLASSSTYE,
"windowstyle",    FT_INT, //  SF_WINDOWSTYLE,
"windowexstyle",  FT_INT, //  SF_WINDOWEXSTYLE,

"rectangle",      FT_STR, //  SF_RECTANGLE,
"restored",       FT_STR, //  SF_RESTORED,
"client",         FT_STR, //  SF_CLIENT,

"threadid",       FT_INT, //  SF_THREADID,
"processid",      FT_INT, //  SF_PROCESSID,

//  SF_LAST
};

BOOL GetWindowInfoStr(struct SpyWindowInfo* info, EnumSpyFormats id, char* result, int maxlen)
{
  if (!info || !result)
    return FALSE;

  BOOL recognized = TRUE;

  switch (id)
  {
  case SF_HWND:
    wsprintf(result, "%08X", info->hwnd);
    break;

  case SF_PARENT:
  case SF_OWNER:
  case SF_HWNDNEXT:
  case SF_HWNDFIRST:
  case SF_HWNDLAST:
  case SF_HWNDPREV:
  case SF_CHILD:
    wsprintf(result, "%08X", info->dependsWindows[id - SF_PARENT]);
    break;

  case SF_WINDOWTEXT:
    lstrcpyn(result, info->windowText, maxlen);
    break;
  case SF_CLASSNAME:
    lstrcpyn(result, info->className, maxlen);
    break;

  case SF_CLASSSTYE:
    wsprintf(result, "%08X", info->classStyle);
    break;
  case SF_WINDOWSTYLE:
    wsprintf(result, "%08X", info->windowStyle);
    break;
  case SF_WINDOWEXSTYLE:
    wsprintf(result, "%08X", info->windowExStyle);
    break;

  case SF_RECTANGLE:
  case SF_RESTORED:
  case SF_CLIENT:
    {
      RECT rect = info->rects[id - SF_RECTANGLE];
      wsprintf(result, "(%d,%d)-(%d,%d), %dx%d",
        rect.left, rect.top, rect.right, rect.bottom,
        rect.right - rect.left, rect.bottom - rect.top);
      break;
    }
  case SF_THREADID:
    wsprintf(result, "%08X", info->ThreadID);
    break;
  case SF_PROCESSID:
    wsprintf(result, "%08X", info->ProcessID);
    break;
  default:
    recognized = FALSE;
  }

  return (recognized);
}

CSpyWindows::CSpyWindows(void)
{
  MaxLevel = 0;

  ItemsCount = 0;
  AllocItemsCount = 0;
  spyWindows = NULL;
}

CSpyWindows::~CSpyWindows(void)
{
  delete[] spyWindows;
  spyWindows = NULL;
}

BOOL CSpyWindows::FindNextLevel(int from, int level)
{
  for (from++; from < ItemsCount; from++) 
  {
    SpyWindow w = spyWindows[from];
    if (w.Level == level) 
      return TRUE;
    if (w.Level < level) 
      break;
  }
  return FALSE;
}

int CSpyWindows::GetTreeChar(int num, int pos, int level)
{
  BOOL next;
  if ((next = FindNextLevel(num, pos + 1)) == TRUE && pos == level - 1) 
    return WTC_NextLink;
  if (pos == level - 1) 
    return WTC_Link;
  return (next) ? WTC_Next : WTC_Null;
}

void CSpyWindows::MkWindows(HWND hwnd, int level)
{
  HWND p;

  MaxLevel = max(level, MaxLevel);

  if (EnumerateHiden || IsWindowVisible(hwnd)) 
  {
    ItemsCount++;
    if (ItemsCount > AllocItemsCount)
    {
      AllocItemsCount += 1024;
      spyWindows = (SpyWindow*)realloc(spyWindows, (AllocItemsCount) * sizeof(SpyWindow));
    }

    spyWindows[ItemsCount - 1].Index = ItemsCount - 1;
    spyWindows[ItemsCount - 1].Level = level;
    spyWindows[ItemsCount - 1].Window = hwnd;
  }

  if ((p = GetWindow(hwnd, GW_CHILD)) != NULL)
    MkWindows(p, level + 1);
  
  if ((p = GetWindow(hwnd, GW_HWNDNEXT)) != NULL)
    MkWindows(p, level);
}

void CSpyWindows::MakeWindowList(BOOL EnumerateHiden)
{
  this->EnumerateHiden = EnumerateHiden;
  
  MaxLevel = 0;
  ItemsCount = 0;
  AllocItemsCount = 0;
  
  delete[] spyWindows;
  spyWindows = NULL;

  MkWindows(GetDesktopWindow(), 0);
}

int CSpyWindows::Count(void)
{
  return ItemsCount;
}

int CSpyWindows::GetMaxLevel(void)
{
  return MaxLevel;
}

struct SpyWindow* CSpyWindows::Items(int index)
{
  if (index >= 0 && index < ItemsCount)
    return &spyWindows[index];
  else
    return NULL;
}

int CSpyWindows::Find(HWND hwnd)
{
  for (int i = 0; i < ItemsCount; i++)
    if (spyWindows[i].Window == hwnd) 
      return i;
  return -1;
}

BOOL CSpyWindows::GetWindowInfo(HWND hwnd, struct SpyWindowInfo* info)
{
  if (!info)
    return FALSE;

  ZeroMemory(info, sizeof(*info));

  if (!IsWindow(hwnd))
    return FALSE;

  info->hwnd = hwnd;
  
  info->dependsWindows[WD_OWNER] = GetWindow(hwnd, GW_OWNER);
  info->dependsWindows[WD_PARENT] = GetParent(hwnd);
  info->dependsWindows[WD_HWNDFIRST] = GetWindow(hwnd, GW_HWNDFIRST);
  info->dependsWindows[WD_HWNDLAST] = GetWindow(hwnd, GW_HWNDLAST);
  info->dependsWindows[WD_HWNDNEXT] = GetWindow(hwnd, GW_HWNDNEXT);
  info->dependsWindows[WD_HWNDPREV] = GetWindow(hwnd, GW_HWNDPREV);
  info->dependsWindows[WD_CHILD] = GetWindow(hwnd, GW_CHILD);

  RECT rect;
  
  if (GetWindowRect(hwnd, &rect))
    info->rects[WR_RECTANGLE] = rect;
  
  if (GetClientRect(hwnd, &rect))
    info->rects[WR_CLIENT] = rect;
  
  WINDOWPLACEMENT wndpl;
  wndpl.length = sizeof(WINDOWPLACEMENT);
  if (GetWindowPlacement(hwnd, &wndpl))
    info->rects[WR_RESTORED] = wndpl.rcNormalPosition;

  DWORD res;
  if (SendMessageTimeout(hwnd, WM_GETTEXT, 
    sizeof(info->windowText), (int)&info->windowText[0], 
    SMTO_ABORTIFHUNG, 1000, &res))
  {
    CharToOemBuff(info->windowText, info->windowText, lstrlen(info->windowText));
  }
  
  GetClassName(hwnd, info->className, sizeof(info->className));
  CharToOemBuff(info->className, info->className, lstrlen(info->className));
  
  info->classStyle = GetClassLong(hwnd, GCL_STYLE);
  
  info->windowStyle = GetWindowLong(hwnd, GWL_STYLE);
  
  info->windowExStyle = GetWindowLong(hwnd, GWL_EXSTYLE);
  
  info->ThreadID = GetWindowThreadProcessId(hwnd, NULL);
  
  static LPDWORD shdw = NULL;
  if (!shdw) 
    shdw = (LPDWORD)GlobalAlloc(GMEM_SHARE, sizeof(DWORD));
  if (!GetWindowThreadProcessId(hwnd, shdw)) 
    *shdw = 0;
  info->ProcessID = *shdw;

  return TRUE;
}

char* CSpyWindows::GetFormatString(HWND hwnd, const char* format, char* res, int maxlen)
{
  SpyWindowInfo info;
  EnumSpyFormats spyFormat;
  char *Result = new char[maxlen];

  GetWindowInfo(hwnd, &info);
  
  lstrcpyn(Result, format, maxlen);

  for (spyFormat = SF_FIRST; spyFormat < SF_LAST; spyFormat = (EnumSpyFormats) (spyFormat+1))
  {
    char OldPattern[MAX_SPYFORMAT + 3];
    char windowInfoStr[MAXSIZE];
    
    if (GetWindowInfoStr(&info, spyFormat, windowInfoStr, sizeof(windowInfoStr)))
    {
      wsprintf(OldPattern, "%%%s%%", spyFormats[spyFormat].name);
      if (!StringReplace(Result, OldPattern, windowInfoStr, maxlen))
      {
        delete [] Result;
        return NULL;
      }
    }
  }
  
  lstrcpyn(res, Result, maxlen);

  delete [] Result;
  return res;
}
